@@ -141,7 +141,7 @@ pub trait TypedIterable {
             return Vec::new();
         }
         let packet = raw.packet;
-        let mut name = Compress::raw_name_to_str(&packet, offset);
+        let mut name = Compress::raw_name_to_str(packet, offset);
         name.make_ascii_lowercase();
         name
     }
@@ -332,6 +332,12 @@ pub trait TypedIterable {
     }
 }
 
+/// Raw RR data.
+pub enum RawRRData<'t> {
+    IpAddr(IpAddr),
+    Data(&'t [u8]),
+}
+
 pub trait RdataIterable {
     /// Returns the TTL for the current RR.
     #[inline]
@@ -359,6 +365,19 @@ pub trait RdataIterable {
         BigEndian::read_u16(&self.rdata_slice()[DNS_RR_RDLEN_OFFSET..]) as usize
     }
 
+    /// Returns the raw record data for the current RR.
+    fn rr_rd(&self) -> Result<RawRRData, Error>
+    where
+        Self: DNSIterable + TypedIterable,
+    {
+        if let Ok(ip_addr) = self.rr_ip() {
+            return Ok(RawRRData::IpAddr(ip_addr));
+        }
+        let rdata_len = self.rr_rdlen();
+        let rdata = &self.rdata_slice()[DNS_RR_HEADER_SIZE..DNS_RR_HEADER_SIZE + rdata_len];
+        Ok(RawRRData::Data(rdata))
+    }
+
     /// Retrieves the IP address of an `A` or `AAAA` record.
     fn rr_ip(&self) -> Result<IpAddr, Error>
     where
@@ -447,8 +466,8 @@ impl<'t> RRIterator<'t> {
         let offset = self
             .offset
             .expect("recompute() called prior to iterating over RRs");
-        let name_end = Self::skip_name(&self.parsed_packet.packet(), offset);
-        let offset_next = Self::skip_rdata(&self.parsed_packet.packet(), name_end);
+        let name_end = Self::skip_name(self.parsed_packet.packet(), offset);
+        let offset_next = Self::skip_rdata(self.parsed_packet.packet(), name_end);
         self.name_end = name_end;
         self.offset_next = offset_next;
     }
